Package: cvar
Type: Package
Title: Compute Expected Shortfall and Value at Risk for Continuous
        Distributions
Version: 0.6
Authors@R: person(given = c("Georgi", "N."), family = "Boshnakov",
                  role = c("aut", "cre"), 
                  email = "georgi.boshnakov@manchester.ac.uk",
		     comment = c(ORCID = "0000-0003-2839-346X"))
Description: Compute expected shortfall (ES) and Value at Risk (VaR) from a
    quantile function, distribution function, random number generator,
    probability density function, or data.  ES is also known as Conditional
    Value at Risk (CVaR). Virtually any continuous distribution can be
    specified.  The functions are vectorized over the arguments. The
    computations are done directly from the definitions, see e.g. Acerbi and
    Tasche (2002) <doi:10.1111/1468-0300.00091>.  Some support for GARCH models
    is provided, as well.
URL: https://geobosh.github.io/cvar/ (doc),
        https://github.com/GeoBosh/cvar (devel)
BugReports: https://github.com/GeoBosh/cvar/issues
Imports: gbutils, Rdpack (>= 0.8)
Suggests: testthat, fGarch, PerformanceAnalytics
RdMacros: Rdpack
License: GPL (>= 2)
Collate: VaR.R cvar-package.R garch.R
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-16 15:57:19 UTC; georgi
Author: Georgi N. Boshnakov [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2839-346X>)
Maintainer: Georgi N. Boshnakov <georgi.boshnakov@manchester.ac.uk>
Repository: CRAN
Date/Publication: 2025-12-17 06:40:47 UTC
Built: R 4.6.0; ; 2026-04-27 14:19:15 UTC; unix
