# Generated by roxygen2: do not edit by hand

S3method(ES,default)
S3method(ES,matrix)
S3method(ES,numeric)
S3method(VaR,default)
S3method(VaR,matrix)
S3method(VaR,numeric)
S3method(predict,garch1c1)
export(ES)
export(GarchModel)
export(VaR)
export(VaR_cdf)
export(VaR_qf)
export(sim_garch1c1)
importFrom(Rdpack,reprompt)
importFrom(gbutils,cdf2quantile)
importFrom(stats,quantile)
importFrom(stats,runif)
